unit aio16a;

interface

function sbitd (data:longword; bit:word):word;
function DAC(DACnum:word; DACV:double):double;
procedure EEPROM_Enable();
procedure EEPROM_Disable();
procedure EEPROM_Write(addr:word; data:word);
function EEPROM_Read(addr:word):word;
procedure CAL_ADC(M:word;B:word);
procedure CAL_DAC(DACNum:word;M:word);
procedure CALWrite(zAM:word;zAB:word;zDAM:word;zDBM:word);
procedure doCAL();
procedure init();

var
  CardNum: LongWord;
  //BASE: word;
  jumpers : word;
  adrange : word;
  dacBrange : word;
  dacArange : word;

implementation

uses
  AIO167;

//sbitd takes data and bit, then returns 0x81 or 0x01 based on data[bit]
function sbitd (data:longword; bit:word):word;
begin
  //return (data & (1<<bit))?0x81:0x01;
  if ((data shr bit) and $1) = 1 then
        sbitd := $81
  else
        sbitd := $1
end;


//DAC will write to the selected DAC and returns the actual volts
//(as modified for 12-bit D/A)
//DACnum is 0 or 1
//DACV is volts
//double DAC(unsigned char DACnum,double DACV)
function DAC(DACnum:word; DACV:double):double;
var
  count, DACRange, DACMax: Word;
  DACCt: Double;

begin
  if DACnum = 0 then DACRange := dacArange else DACRange := dacBrange;
  if DACRange <> 0 then DACMax := 5 else DACMax := 10;

  DACCt := DACV/DACMax * $1000;
  if DACCt > $FFF then DACCt := $FFF;
  if DACCt < 0 then DACCt := 0;
  count := round(DACCt);

  RelOutPort(CardNum, $C + DACnum * 2, count);//write value to DAC

  DAC := (count/4096.0) * DACMax;
end;

// Enable the EEEPROM for writing
procedure EEPROM_Enable();
begin
  RelOutPortB(CardNum, $18, $80);
  RelOutPortB(CardNum, $18, $81);//start bit
  RelOutPortB(CardNum, $18, $01);//Opcode bit 0 (00 = enable/disable)
  RelOutPortB(CardNum, $18, $01);//Opcode bit 1  ""
  RelOutPortB(CardNum, $18, $81);//a5 (must be 1 to enable)
  RelOutPortB(CardNum, $18, $81);//a4 (must be 1 to enable)
  RelOutPortB(CardNum, $18, $01);//a3 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a2 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a1 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a0 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $00);//end trans

end;

// Disable the EEPROM write
procedure EEPROM_Disable();
begin
  RelOutPortB(CardNum, $18, $80);
  RelOutPortB(CardNum, $18, $81);//start bit
  RelOutPortB(CardNum, $18, $01);//Opcode bit 0 (00 = enable/disable)
  RelOutPortB(CardNum, $18, $01);//Opcode bit 1  ""
  RelOutPortB(CardNum, $18, $01);//a5 (must be 0 to disable)
  RelOutPortB(CardNum, $18, $01);//a4 (must be 0 to disable)
  RelOutPortB(CardNum, $18, $01);//a3 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a2 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a1 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $01);//a0 (don't care because enable/disable)
  RelOutPortB(CardNum, $18, $00);//end trans
end;

// write the value in data to the EEPROM to the location specified by addr
procedure EEPROM_Write(addr:word; data:word);
begin
  RelOutPortB(CardNum, $18, $80);
  RelOutPortB(CardNum, $18, $81);//Start Bit
  RelOutPortB(CardNum, $18, $01);//Opcode Bit 0 (01=write)
  RelOutPortB(CardNum, $18, $81);//Opcode Bit 1  ""
  RelOutPortB(CardNum, $18, sbitd(addr,5));//a5 (address to write to)
  RelOutPortB(CardNum, $18, sbitd(addr,4));//a4
  RelOutPortB(CardNum, $18, sbitd(addr,3));//a3
  RelOutPortB(CardNum, $18, sbitd(addr,2));//a2
  RelOutPortB(CardNum, $18, sbitd(addr,1));//a1
  RelOutPortB(CardNum, $18, sbitd(addr,0));//a0
  RelOutPortB(CardNum, $18, sbitd(data,15));//d15
  RelOutPortB(CardNum, $18, sbitd(data,14));//d14
  RelOutPortB(CardNum, $18, sbitd(data,13));//d13
  RelOutPortB(CardNum, $18, sbitd(data,12));//d12
  RelOutPortB(CardNum, $18, sbitd(data,11));//d11
  RelOutPortB(CardNum, $18, sbitd(data,10));//d10
  RelOutPortB(CardNum, $18, sbitd(data,9));//d9
  RelOutPortB(CardNum, $18, sbitd(data,8));//d8
  RelOutPortB(CardNum, $18, sbitd(data,7));//d7
  RelOutPortB(CardNum, $18, sbitd(data,6));//d6
  RelOutPortB(CardNum, $18, sbitd(data,5));//d5
  RelOutPortB(CardNum, $18, sbitd(data,4));//d4
  RelOutPortB(CardNum, $18, sbitd(data,3));//d3
  RelOutPortB(CardNum, $18, sbitd(data,2));//d2
  RelOutPortB(CardNum, $18, sbitd(data,1));//d1
  RelOutPortB(CardNum, $18, sbitd(data,0));//d0
  RelOutPortB(CardNum, $18, $00);//end trans
end;

// read the value from the EEPROM at the location specified by addr and return it
function EEPROM_Read(addr:word):word;
var
  data:word;
begin
  data:=0;
  RelOutPortB(CardNum, $18, $80);
  RelOutPortB(CardNum, $18, $81);//Start Bit
  RelOutPortB(CardNum, $18, $81);//Opcode Bit 0 (10=Read)
  RelOutPortB(CardNum, $18, $01);//Opcode Bit 1  ""
  RelOutPortB(CardNum, $18, sbitd(addr,5));//a5 (address to write to)
  RelOutPortB(CardNum, $18, sbitd(addr,4));//a4
  RelOutPortB(CardNum, $18, sbitd(addr,3));//a3
  RelOutPortB(CardNum, $18, sbitd(addr,2));//a2
  RelOutPortB(CardNum, $18, sbitd(addr,1));//a1
  RelOutPortB(CardNum, $18, sbitd(addr,0));//a0
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d15
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d14
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d13
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d12
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d11
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d10
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d9
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d8
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d7
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d6
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d5
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d4
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d3
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d2
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d1
  data :=(data shl 1) or ((RelInPortB(CardNum, $18) shr 7));//d0
  RelOutPortB(CardNum, $18, $00);//end com
  EEPROM_Read := data;
end;

// write the gain (M) and the offset (B) to the pots. y=Mx + B
procedure CAL_ADC(M:word;B:word);
begin
 //write cal to A/D, offset first, then slope
  RelOutPortB(CardNum, $19, $80);
  RelOutPortB(CardNum, $19, $01);//a1 pot 00 is offset
  RelOutPortB(CardNum, $19, $01);//a0 ""
  RelOutPortB(CardNum, $19, sbitd(B,7));//data bit 7 of offset
  RelOutPortB(CardNum, $19, sbitd(B,6));//data bit 6 of offset
  RelOutPortB(CardNum, $19, sbitd(B,5));//data bit 5 of offset
  RelOutPortB(CardNum, $19, sbitd(B,4));//data bit 4 of offset
  RelOutPortB(CardNum, $19, sbitd(B,3));//data bit 3 of offset
  RelOutPortB(CardNum, $19, sbitd(B,2));//data bit 2 of offset
  RelOutPortB(CardNum, $19, sbitd(B,1));//data bit 1 of offset
  RelOutPortB(CardNum, $19, sbitd(B,0));//data bit 0 of offset
  RelOutPortB(CardNum, $19, $00);//end trans
  //finished writing offset.
  //the following code is a new, independant transaction on the serial chip
  //write slope now
  RelOutPortB(CardNum, $19, $80);
  RelOutPortB(CardNum, $19, $01);//a1 pot 01 is slope of A/D
  RelOutPortB(CardNum, $19, $81);//a0 pot 01 is slope of A/D
  RelOutPortB(CardNum, $19, sbitd(M,7));//data bit 7 of slope
  RelOutPortB(CardNum, $19, sbitd(M,6));//data bit 6 of slope
  RelOutPortB(CardNum, $19, sbitd(M,5));//data bit 5 of slope
  RelOutPortB(CardNum, $19, sbitd(M,4));//data bit 4 of slope
  RelOutPortB(CardNum, $19, sbitd(M,3));//data bit 3 of slope
  RelOutPortB(CardNum, $19, sbitd(M,2));//data bit 2 of slope
  RelOutPortB(CardNum, $19, sbitd(M,1));//data bit 1 of slope
  RelOutPortB(CardNum, $19, sbitd(M,0));//data bit 0 of slope
  RelOutPortB(CardNum, $19, $00);//end trans
end;

// write the gain(M) for the specified DAC to the pots
procedure CAL_DAC(DACNum:word;M:word);
var PCICARD:integer;//the PCI version has the DAC cal chip at +1A
begin
  if RelInPortB(CardNum, $1F)=0 then
    PCICARD:=1
  else
    PCICARD:=0;

  RelOutPortB(CardNum, $19+PCICARD, $80);
  if (PCICARD=1) then
    RelOutPortB(CardNum, $1A, $01)
  else
    RelOutPortB(CardNum, $19, $81)
  ;
  RelOutPortB(CardNum, $19+PCICARD, sbitd(DACNum,0));//a0 pot 11 is slope of DAC1
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,7));//data bit 7 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,6));//data bit 6 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,5));//data bit 5 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,4));//data bit 4 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,3));//data bit 3 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,2));//data bit 2 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,1));//data bit 1 of slope
  RelOutPortB(CardNum, $19+PCICARD, sbitd(M,0));//data bit 0 of slope
  RelOutPortB(CardNum, $19+PCICARD, $00);//end trans
end;

// writes the current calibration values to the EEPROM.
procedure CALWrite(zAM:word;zAB:word;zDAM:word;zDBM:word);
begin
 EEPROM_Enable();
 EEPROM_Write((adrange+0),zAB);
 EEPROM_Write((adrange+8),zAM);
 EEPROM_Write((dacArange+$10),zDAM);
 EEPROM_Write((dacBrange+$12),zDBM);
 EEPROM_Disable();

end;

// reads the calibration values from the EEPROM and write them to the pots
// this should be done before taking any readings from the board.


procedure doCAL();

var
zDAM : word;
zDBM : word;
zAM : word;
zAB : word;

begin

  zAB := EEPROM_Read((adrange+0))and $ff;  //byte 0-7 in EEPROM holds "B" for ADC
  zAM := EEPROM_Read((adrange+8))and $ff;  //byte 8-f in EEPROM holds "M" for ADC
  zDAM := EEPROM_Read((dacArange+$10))and $ff; //byte x10,x11 in EEPROM holds "M" for DAC0
  zDBM := EEPROM_Read((dacBrange+$12))and $ff; //byte x12,x13 in EEPROM holds "M" for DAC1
  CAL_DAC(0,zDAM);//write the calibration value from EEPROM to CAL Pots
  CAL_DAC(1,zDBM);// ""
  CAL_ADC(zAM,zAB);// ""

end;

//reads the jumper config from the board and initializes the global variables

procedure init();
begin
  jumpers := RelInPortB(CardNum, $12);//jumper configuration readback
  adrange := jumpers and $07;//d2,d1,d0 are 5/10, 16/8, uni/bip
  dacBrange := (jumpers shr 3) and $01;//d4,d3 are 5/10 for daca,b
  dacArange := (jumpers shr 4) and $01;
//  CAL();
end;

end.
