unit Sample1u;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls;

const
  AM_READDATA = WM_USER + 71;

type
  TSampleForm = class(TForm)
    LogMemo: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    SecondsEdit: TEdit;
    Label2: TLabel;
    FileEdit: TEdit;
    BeginBTN: TButton;
    Panel2: TPanel;
    WaitingButton: TButton;
    procedure BeginBTNClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure WaitingButtonClick(Sender: TObject);
  protected
    procedure AMReadData(var Msg: TMessage); message AM_READDATA;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SampleForm: TSampleForm;
  channels: integer;
  dac0mult, dac1mult: double;

implementation

{$R *.DFM}

uses
  AIO167, AIO16A;

const
  FIFOHalfWords = 512;
  FIFOHalfBytes = 2 * FIFOHalfWords;
  NO_CARD = $FFFFFFFF;

type
  TAcqThread = class(TThread)
  public
    BytesLeftToTake: Integer;
    bWaiting: Boolean;
  protected
    procedure Execute; override;
  end;

var
  ADData: array of Word;
  AcqThread: TAcqThread = nil;
  LogFile: TFileStream;

procedure CtrMode(CardNum: LongWord; cntr, mode: Byte);
var
  ctrl: byte;

begin
  ctrl := (cntr shl 6) or $30 or (mode shl 1);
  RelOutPortB(CardNum, 3, ctrl);
end;

procedure CtrLoad(CardNum: LongWord; c, val: Word);
begin
  RelOutPortB(CardNum, c, lo(val));
  RelOutPortB(CardNum, c, hi(val));
end;

{ TSampleForm }

procedure TSampleForm.FormCreate(Sender: TObject);
var
  I, NumCards: Integer;
  Status, Buf: LongWord;

begin
  CardNum := NO_CARD;
  NumCards := GetNumCards;
  for I := 0 to NumCards - 1 do begin
    Status := QueryCardID(I, @Buf);
    if Status <> ERROR_SUCCESS then Continue; //Failed to query ID.
    if Buf <> $1ECE8 then Continue; //Not the ID of the 104-AIO16E.
    Status := QueryCardBase(I, @Buf);
    if Status <> ERROR_SUCCESS then Continue;

    //We've successfully queried it and found the card we're looking for.
    CardNum := I;
    Break;
  end;

  if CardNum = NO_CARD then begin
    LogMemo.Text := 'No 104-AIO16E found; make sure it''s installed in Device Manager.'#13#10;
  end
  else begin
    LogMemo.Text := '104-AIO16E found as card number ' + IntToStr(CardNum) + '. Click Begin to begin acquisition.'#13#10;
  end;
end;

procedure TSampleForm.BeginBTNClick(Sender: TObject);
var
  Size: Integer;

begin
    if CardNum = NO_CARD then begin
      Beep;
      LogMemo.Font.Color := $FFFFFF;
      LogMemo.Color := $0000FF;
      Exit;
    end;

    if AcqThread <> nil then begin
      Beep;
      Exit;
    end;



    //Set buffer size based on seconds to acquire.
    Size := StrToInt(SecondsEdit.Text);
    SetLength(ADData, Size * 160000); //The counter load values below take 160k samples per second.

    //Open the file we'll save the data to.
    LogFile := TFileStream.Create(FileEdit.Text, fmCreate);

    //Initialize the global variables and then load the card's calibration.
    init;
    doCAL;

    //tell the card to take one reading per channel using Scan Start
    RelOutPortB(CardNum, $11, $05);//Set ADC SCAN MODE and COUNTER START

    //Both A/D counters in mode 2; they'll be loaded below.
    CtrMode(CardNum, 1, 2);
    CtrMode(CardNum, 2, 2);

    //determine if card is in single ended or differential mode
    if (jumpers and $2) = 2 then begin
        channels := 16;
        RelOutPortB(CardNum, $06, $F0); //scan all channels, zero to fifteen

        //Ctr 1 & 2 divisor of 1000 with 10MHz clock gives 10kHz tick.
        //Scanning 16 channels, that's 160k samples per second.
        CtrLoad(CardNum, 1, 2);
        CtrLoad(CardNum, 2, 500);
    end
    else begin
        channels := 8;
        RelOutPortB(CardNum, $06, $70); //scan all channels, zero to seven

        //Ctr 1 & 2 divisor of 500 with 10MHz clock gives 20kHz tick.
        //Scanning 8 channels, that's 160k samples per second.
        CtrLoad(CardNum, 1, 2);
        CtrLoad(CardNum, 2, 250);
    end;

    //Launch the acquisition thread.
    AcqThread := TAcqThread.Create(True);
    AcqThread.BytesLeftToTake := Length(ADData) * SizeOf(ADData[0]);
    AcqThread.Resume;

    LogMemo.Lines.Add('Acquiring...');
end;

procedure TSampleForm.AMReadData(var Msg: TMessage);
var
  Content, SampleStr: String;
  I: Integer;

begin
  if AcqThread.ReturnValue <> Msg.WParam then begin
    LogMemo.Lines.Add('Return value mismatch; ' + IntToStr(AcqThread.ReturnValue) + ' <> ' + IntToStr(Msg.WParam) + '.');
  end
  else if AcqThread.ReturnValue <> ERROR_SUCCESS then begin
    LogMemo.Lines.Add('Error ' + IntToStr(AcqThread.ReturnValue) + ' from WaitForIRQData.');
  end
  else begin
    LogMemo.Lines.Add('Acquisition complete, saving...');
    SetString(Content, nil, Length(ADData) * 6);
    for I := 0 to High(ADData) do begin
      SampleStr := IntToHex(ADData[I], 4) + #13#10;
      Move(SampleStr[1], Content[1 + 6*I], 6);
    end;
    LogFile.Write(Content[1], Length(Content));
    FreeAndNil(LogFile);
    LogMemo.Lines.Add('Saving complete.');
  end;
end;

{ TAcqThread }

procedure TAcqThread.Execute;
var
  BytesThisFIFO: Integer;
  pNextAD: PByte;

begin
  ReturnValue := ERROR_SUCCESS;
  pNextAD := PByte(@ADData[0]);
  while BytesLeftToTake > 0 do begin
    BytesThisFIFO := BytesLeftToTake;
    if BytesThisFIFO > FIFOHalfBytes then BytesThisFIFO := FIFOHalfBytes;

    bWaiting := True;
    ReturnValue := WaitForIRQData(CardNum, BytesThisFIFO, pNextAD);
    bWaiting := False;
    if ReturnValue <> ERROR_SUCCESS then begin
      PostMessage(SampleForm.Handle, AM_READDATA, ReturnValue, 0);
      Exit;
    end;

    //At this point, another thread could process the part we've taken, but we'll wait for acquisition to finish because string processing is slow.

    Inc(pNextAD, BytesThisFIFO);
    Dec(BytesLeftToTake, BytesThisFIFO);
  end;

  PostMessage(SampleForm.Handle, AM_READDATA, ReturnValue, 0);
end;

procedure TSampleForm.WaitingButtonClick(Sender: TObject);
begin
  if AcqThread = nil then
    WaitingButton.Caption := 'Is It Waiting'
  else if AcqThread.bWaiting then
    WaitingButton.Caption := 'Yes'
  else
    WaitingButton.Caption := 'No'
  ;
end;

end.
