
// AQSDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AQS.h"
#include "AQSDlg.h"
#include "afxdialogex.h"

#include "TIRQThread.h"

#include "AIO167.h"
#include "AIO16A.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define FIFOHalfWords 512
#define FIFOHalfBytes (2 * FIFOHalfWords)
const UINT32 NO_CARD = 0xFFFFFFFF;

UINT32 QUADCardNum;
UINT32 QUADBase;
int ComNum;
TIRQThread * AcqThread;


//Quadrature stuff.
#define RLD 0x80
#define XRLD 0x00
#define YRLD XRLD
#define Rst_BP 0x01
#define Rst_CTR 0x02
#define Rst_FLAGS 0x04
#define Rst_E 0x06
#define Trf_PR_CTR 0x08
#define Trf_CTR_OL 0x10
#define Trf_PS0_PSC 0x18

#define CMR 0xA0
#define XCMR 0x20
#define YCMR XCMR
#define BINCnt 0x00
#define BCDCnt 0x01
#define NrmCnt 0x00
#define RngLmt 0x02
#define NRcyc 0x04
#define ModN 0x06
#define NQDX 0x00
#define QDX1 0x08
#define QDX2 0x10
#define QDX4 0x18

#define IOR 0xC0
#define XIOR 0x40
#define YIOR 0x00
#define DisAB 0x00
#define EnAB 0x01
#define LCTR 0x00
#define LOL 0x02
#define RCTR 0x00
#define ABGate 0x04
#define CYBW 0x00
#define CPBW 0x08
#define CB_UPDN 0x10
#define IDX_ERR 0x18

#define IDR 0xE0
#define XIDR 0x60
#define YIDR XIDR
#define DisIDX 0x00
#define EnIDX 0x01
#define NIDX 0x00
#define PIDX 0x02
#define LIDX 0x00
#define RIDX 0x04


#define PI 3.141593

#define TIMER_DELAY 200

CString IntToStr(int I)
{
	CString Result;
	Result.Format("%d", I);
	return Result;
}


// CAboutDlg dialog used for App About

class CAboutDlg : public CDialogEx
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialogEx(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialogEx)
END_MESSAGE_MAP()


// CAQSDlg dialog




CAQSDlg::CAQSDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CAQSDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_StatusString = _T("");
}

void CAQSDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_STATUS, m_StatusControl);
	DDX_Text(pDX, IDC_EDIT_STATUS, m_StatusString);
}

BEGIN_MESSAGE_MAP(CAQSDlg, CDialogEx)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_STOP, &CAQSDlg::OnBnClickedButtonStop)
	ON_BN_CLICKED(IDC_BUTTON_STARTAD, &CAQSDlg::OnBnClickedButtonStartad)
	ON_BN_CLICKED(IDC_BUTTON_STARTQUAD, &CAQSDlg::OnBnClickedButtonStartquad)
	ON_MESSAGE(0x0477, &CAQSDlg::OnThreadEnd)
END_MESSAGE_MAP()


// CAQSDlg message handlers

BOOL CAQSDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		BOOL bNameValid;
		CString strAboutMenu;
		bNameValid = strAboutMenu.LoadString(IDS_ABOUTBOX);
		ASSERT(bNameValid);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon


	UINT32 Buf;

	AIO16CardNum = NO_CARD;
	QUADCardNum = NO_CARD;
	ComNum = 0;

	int NumCards = GetNumCards();
	for ( int I = 0; I < NumCards; ++I )
	{
		UINT32 Status = QueryCardID(I, &Buf);
		if ( Status != ERROR_SUCCESS ) continue; //Failed to query ID.
		switch( Buf )
		{
			case 0x1ECE8: if ( AIO16CardNum == NO_CARD ) //104-AIO16E
			{
				Status = QueryCardBase(I, &Buf);
				if ( Status != ERROR_SUCCESS ) continue;

				AIO16CardNum = I;
			break;
			}
			case 0x12230: if ( QUADCardNum == NO_CARD ) //104-QUAD-8
			{
				Status = QueryCardBase(I, &QUADBase);
				if ( Status != ERROR_SUCCESS ) continue;

				QUADCardNum = I;
			break;
			}
			default: continue;
		}

		if ((AIO16CardNum != NO_CARD) && (QUADCardNum != NO_CARD)) break;
	}

	for ( int I = 1; I <= 255; ++I )
	{
		CString ComBuf;
		ComBuf.Format("\\\\.\\COM%d", I);
		HANDLE hCom = CreateFile(ComBuf, GENERIC_WRITE | GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
		ComBuf = "";
		if ( hCom != INVALID_HANDLE_VALUE )
		{
			ComNum = I;
			CloseHandle(hCom);
			break;
		}
	}

	if ( AIO16CardNum == NO_CARD )
		m_StatusString += "No 104-AIO16E found; make sure it's installed in Device Manager.\r\n";
	else
		m_StatusString += "104-AIO16E found as card number " + IntToStr(AIO16CardNum) + ".\r\n"
	;
	m_StatusString += "\r\n";

	if ( QUADCardNum == NO_CARD )
		m_StatusString += "No 104-QUAD-8 found; make sure it's installed in Device Manager.\r\n";
	else
		m_StatusString += "104-QUAD-8 found as card number " + IntToStr(QUADCardNum) + ".\r\n"
	;
	m_StatusString += "\r\n";

	if ( ComNum == 0 )
		m_StatusString += "No serial port found; make sure one is installed.\r\n";
	else
		m_StatusString += "Serial port found at COM" + IntToStr(ComNum) + ".\r\n"
	;
	m_StatusString += "\r\n";

	if ( (AIO16CardNum != NO_CARD) && (QUADCardNum != NO_CARD) && (ComNum != 0) )
		m_StatusString += "Click Start to start acquisition.\r\n\r\n"
	;


	UpdateData(false);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CAQSDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialogEx::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CAQSDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CAQSDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CtrMode(UINT32 CardNum, unsigned char cntr, unsigned char mode)
{
	RelOutPortB(CardNum, 8+3, (cntr << 6) | 0x30 | (mode << 1));
}

void CtrLoad(UINT32 CardNum, unsigned char c, unsigned short val)
{
	RelOutPortB(CardNum, 8+c, val & 0xFF);
	RelOutPortB(CardNum, 8+c, (val>>8) & 0xFF);
}

void CAQSDlg::OnBnClickedButtonStartad()
{
	OnBnClickedButtonStart(false);
}


void CAQSDlg::OnBnClickedButtonStartquad()
{
	OnBnClickedButtonStart(true);
}

void CAQSDlg::OnBnClickedButtonStart(bool bReportQuad)
{
	if ((AIO16CardNum == NO_CARD) || (QUADCardNum == NO_CARD) || (ComNum == 0))
	{
		MessageBeep(0);
		m_StatusString += "\r\nCard or COM port Missing!\r\n";
		UpdateData(false);
		return;
	}

	if ( AcqThread != NULL )
	{
		MessageBeep(0);
		m_StatusString += "\r\nThread already running!\r\n";
		UpdateData(false);
		return;
	}


	//Reset entire board to clear out any previous config.
	RelOutPortB(AIO16CardNum, 0x1B, 0xFF);

	//Initialize the global variables and then load the card's calibration.
	init();
	doCAL();

	//tell the card to take one reading per channel using Scan Start
	RelOutPortB(AIO16CardNum, 0x11, 0x05);//Set ADC SCAN MODE and COUNTER START

	RelOutPortB(AIO16CardNum, 0x13, 0x02); //Enable IRQ on end-of-scan.

	//Both A/D counters in mode 2; they'll be loaded below.
	CtrMode(AIO16CardNum, 1, 2);
	CtrMode(AIO16CardNum, 2, 2);

	RelOutPortB(AIO16CardNum, 0x06, 0xF0); //scan all channels, zero to fifteen

	//Ctr 1 & 2 divisor of 166666 with 10MHz clock is close to 60Hz tick.
	CtrLoad(AIO16CardNum, 1, 167);
	CtrLoad(AIO16CardNum, 2, 998);


	//Init the quad board.
	Init_7266(QUADBase);


	//Launch the acquisition thread.
	AcqThread = (TIRQThread*)AfxBeginThread(RUNTIME_CLASS(TIRQThread), THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
	AcqThread->SetOwner(this);
	AcqThread->bTerminated = false;
	AcqThread->hMainForm = m_hWnd;
	AcqThread->bReportQuad = bReportQuad;
	AcqThread->ResumeThread();

	m_StatusString += "\r\nAcquiring...\r\n";
	UpdateData(false);
}


void CAQSDlg::OnBnClickedButtonStop()
{
	if ( AcqThread )
	{
		m_StatusString += "\r\nStopping...\r\n";
		UpdateData(false);
		AcqThread->bTerminated = true;
	}
}

afx_msg LRESULT CAQSDlg::OnThreadEnd(WPARAM wParam, LPARAM lParam)
{
	if ( AcqThread )
	{
		m_StatusString += "\r\nStopped.\r\n";
		UpdateData(false);
		AcqThread->Delete();
		AcqThread = NULL;
	}

	return 0;
}



void Init_7266(unsigned short Addr)
{
	unsigned long XCt;
	unsigned long YCt;

	for (int count = 0; count < 4; count++)
	{
		XCt = 0xFFFFFF;
		YCt = 0xFFFFFF;

		//Setup IOR reg
		OutPortB(Addr + 1, IOR + DisAB + LOL + ABGate + CYBW);

		//Setup RLD reg
		OutPortB(Addr + 1, RLD + Rst_BP + Rst_FLAGS);
		OutPortB(Addr + 0, 0x0);
		OutPortB(Addr + 2, 0x0);
		OutPortB(Addr + 1, RLD + Rst_E + Trf_PS0_PSC);
		OutPortB(Addr + 1, RLD + Rst_BP + Rst_CTR);

		//Setup IDR reg
		OutPortB(Addr + 1, IDR + EnIDX + NIDX + LIDX);

		//Setup CMR reg
		OutPortB(Addr + 1, CMR + BINCnt + ModN + QDX1);

		//Setup PR reg for modulo N counter to XCt
		OutPortB(Addr + 0, UINT8(XCt));
		XCt >>= 8;
		OutPortB(Addr + 0, UINT8(XCt));
		XCt >>= 8;
		OutPortB(Addr + 0, UINT8(XCt));

		//Setup PR reg for modulo N counter to YCt
		OutPortB(Addr + 2, UINT8(YCt));
		YCt >>= 8;
		OutPortB(Addr + 2, UINT8(YCt));
		YCt >>= 8;
		OutPortB(Addr + 2, UINT8(YCt));

		//Enable counters
		OutPortB(Addr + 1, IOR + EnAB);

		Addr += 4;
	}
}

int Read_7266_OL(unsigned short Addr)
{
	union pos_tag {		/* allows access of 32-bit integer as 4 bytes */
		int l;
		struct byte_tag {char b0; char b1; char b2; char b3;} byte;
	}
	pos;

  	OutPortB(Addr + 1, 0x10);	/* reset address pointer */
	OutPortB(Addr + 1, 0x01);	/* command to latch counter */
	pos.byte.b0 = UINT8(InPortB(Addr));
	pos.byte.b1 = UINT8(InPortB(Addr));
	pos.byte.b2 = UINT8(InPortB(Addr));
	/* extend sign of position */
	if (pos.byte.b2 < 0) pos.byte.b3 = -1;
	else pos.byte.b3 = 0;
	
	return pos.l;

}
