// Control.cpp -- IOCTL handlers for AIO16 driver

#include "stddcls.h"
#include "driver.h"
#include "ioctls.h"

void ForgetIRP(PIRP IRP);

NTSTATUS CacheControlRequest(PDEVICE_EXTENSION pdx, KSPIN_LOCK *Lock, PIRP Irp, PIRP* pIrp);
VOID OnCancelPendingIoctl(PDEVICE_OBJECT fdo, PIRP Irp);
NTSTATUS OnCompletePendingIoctl(PDEVICE_OBJECT junk, PIRP Irp, PDEVICE_EXTENSION pdx);
VOID IRQGenCancelIrp(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);

///////////////////////////////////////////////////////////////////////////////

#define AIO16NT_PACKETCHECK \
	if ( InBufferSize < sizeof(AIO16NT_BUFFER) || OutBufferSize < sizeof(AIO16NT_BUFFER) ) {\
	Info = 0; Status = STATUS_INVALID_PARAMETER; break; }
//Don't return anything if check fails

#pragma PAGEDCODE

NTSTATUS DispatchControl(PDEVICE_OBJECT fdo, PIRP Irp)
{
	PDEVICE_EXTENSION pDX = (PDEVICE_EXTENSION) fdo->DeviceExtension;

	NTSTATUS Status = IoAcquireRemoveLock(&pDX->RemoveLock, Irp);
	if (!NT_SUCCESS(Status))
		return CompleteRequest(Irp, Status, 0);
	ULONG Info = 0;

	PIO_STACK_LOCATION stack = IoGetCurrentIrpStackLocation(Irp);
	ULONG InBufferSize = stack->Parameters.DeviceIoControl.InputBufferLength;
	ULONG OutBufferSize = stack->Parameters.DeviceIoControl.OutputBufferLength;
	ULONG code = stack->Parameters.DeviceIoControl.IoControlCode;
	PVOID pDataBuffer;

	Irp->IoStatus.Information = 0;

	switch (code)
	{

	case IOCTL_PORT_IN:
		{

			if ( (InBufferSize < sizeof(TPORTOP)) || OutBufferSize < sizeof(TPORTOP) )
			{
				Status = STATUS_INVALID_PARAMETER;
				break;
			}
			PPORTOP POBuf = PPORTOP(Irp->AssociatedIrp.SystemBuffer);
			if ( POBuf->Flags & POF_RELATIVE ) POBuf->Address += (USHORT)pDX->Base;
			switch(POBuf->Size)
			{
				case 1:
					POBuf->Data8 = READ_PORT_UCHAR(PUCHAR(UNATIVE(POBuf->Address)));
				break;
				case 2:
					POBuf->Data16 = READ_PORT_USHORT(PUSHORT(UNATIVE(POBuf->Address)));
				break;
				case 4:
					POBuf->Data32 = READ_PORT_ULONG(PULONG(UNATIVE(POBuf->Address)));
				break;
			}
			Info = sizeof(TPORTOP);

			Status = STATUS_SUCCESS;

		break;
		}

	case IOCTL_PORT_OUT:
		{

			if ( (InBufferSize < sizeof(TPORTOP)) )
			{
				Status = STATUS_INVALID_PARAMETER;
				break;
			}
			PPORTOP POBuf = PPORTOP(Irp->AssociatedIrp.SystemBuffer);
			if ( POBuf->Flags & POF_RELATIVE ) POBuf->Address += (USHORT)pDX->Base;
			switch(POBuf->Size)
			{
				case 1:
					WRITE_PORT_UCHAR(PUCHAR(UNATIVE(POBuf->Address)), UCHAR(POBuf->Data8));
				break;
				case 2:
					WRITE_PORT_USHORT(PUSHORT(UNATIVE(POBuf->Address)), USHORT(POBuf->Data16));
				break;
				case 4:
					WRITE_PORT_ULONG(PULONG(UNATIVE(POBuf->Address)), ULONG(POBuf->Data32));
				break;
			}
			Info = 0;

			Status = STATUS_SUCCESS;

		break;
		}

	case IOCTL_PORT_IN_STRING:
		{

			if ( (InBufferSize < sizeof(TPORTOP)) || OutBufferSize < 1 )
			{
				Status = STATUS_INVALID_PARAMETER;
				break;
			}
			PPORTOP POBuf = PPORTOP(Irp->AssociatedIrp.SystemBuffer);
			if ( POBuf->Flags & POF_RELATIVE ) POBuf->Address += (USHORT)pDX->Base;
			Status = STATUS_SUCCESS;
			switch(POBuf->Size)
			{
				case 1:
					Info = OutBufferSize & 0xFFFF;
					if ( Info == 0 )
						Status = STATUS_INVALID_PARAMETER;
					else
						READ_PORT_BUFFER_UCHAR(PUCHAR(UNATIVE(POBuf->Address)), PUCHAR(Irp->AssociatedIrp.SystemBuffer), Info)
					;
				break;
				case 2:
					Info = OutBufferSize & 0xFFFE;
					if ( Info == 0 )
						Status = STATUS_INVALID_PARAMETER;
					else
						READ_PORT_BUFFER_USHORT(PUSHORT(UNATIVE(POBuf->Address)), PUSHORT(Irp->AssociatedIrp.SystemBuffer), Info >> 1)
					;
				break;
				case 4:
					Info = OutBufferSize & 0xFFFC;
					if ( Info == 0 )
						Status = STATUS_INVALID_PARAMETER;
					else
						READ_PORT_BUFFER_ULONG(PULONG(UNATIVE(POBuf->Address)), PULONG(Irp->AssociatedIrp.SystemBuffer), Info >> 2)
					;
				break;
				default:
					Info = 0;
					Status = STATUS_INVALID_PARAMETER;
				break;
			}

		break;
		}

	case IOCTL_IRQ_WAIT_BEGIN:
		{

			if ( pDX->IRQState != isNoIRQ )
			{
				Status = STATUS_INVALID_DEVICE_REQUEST;
				break;
			}

			ULONG bHistorical = InterlockedExchange(PLONG(&(pDX->InterruptOccurred)), FALSE);
			if ( bHistorical )
			{
				Info = (OutBufferSize & 0xFFFE) >> 1;
				READ_PORT_BUFFER_USHORT(PUSHORT(UNATIVE(pDX->Base + 0x00)), PUSHORT(Irp->AssociatedIrp.SystemBuffer), Info);
				Status = STATUS_SUCCESS;
			}
			else
			{
				pDX->ADCount = (OutBufferSize & 0xFFFE) >> 1;
				pDX->PendingIRP = Irp;
				IoMarkIrpPending(Irp);
				IoSetCancelRoutine(Irp, IRQGenCancelIrp);
				pDX->IRQState = isWaitingForIRQ;
				Status = STATUS_PENDING; // do not complete Irp of course
			}

		break;
		}

	case IOCTL_IRQ_WAIT_ABORT:
		{

			if ( pDX->IRQState != isWaitingForIRQ )
				Status = STATUS_NO_SUCH_FILE;
			else
			{
				pDX->InterruptOccurred = FALSE;
				pDX->IRQState = isNoIRQ;
				pDX->PendingIRP->IoStatus.Status = STATUS_CANCELLED;
				pDX->PendingIRP->IoStatus.Information = 0;
				IoSetCancelRoutine(pDX->PendingIRP, NULL);
				IoCompleteRequest(pDX->PendingIRP, IO_SERIAL_INCREMENT);
				pDX->PendingIRP = NULL;
				Status = STATUS_SUCCESS;
			}

		break;
		}

	case IOCTL_FORGET_IRP:
		{

			if ( pDX->IRQState == isNoIRQ )
				Status = STATUS_NO_SUCH_FILE;
			else
			{
				pDX->IRQState = isNoIRQ;
				ForgetIRP(pDX->PendingIRP);
				pDX->PendingIRP = NULL;
				Status = STATUS_SUCCESS;
			}

		break;
		}

	default:
		Info = 0;
		Status = STATUS_INVALID_DEVICE_REQUEST;
		break;

	}

	IoReleaseRemoveLock(&pDX->RemoveLock, Irp);
	if ( Status != STATUS_PENDING )
	{
		Irp->IoStatus.Information = Info;
		Status = CompleteRequest(Irp, Status, Info);
	}
	return Status;
}



///////////////////////////////////////////////////////////////////////////////

#pragma PAGEDCODE

VOID AbortPendingIoctls(PDEVICE_EXTENSION pdx, NTSTATUS status)
	{							// AbortPendingIoctls
	PAGED_CODE();
	InterlockedExchange(&pdx->IoctlAbortStatus, status);
	CleanupControlRequests(pdx, status, NULL);
	}							// AbortPendingIoctls

///////////////////////////////////////////////////////////////////////////////

#pragma LOCKEDCODE

NTSTATUS CacheControlRequest(PDEVICE_EXTENSION pdx, KSPIN_LOCK *Lock, PIRP Irp, PIRP *pIrp)
	{							// CacheControlRequest
	ASSERT(KeGetCurrentIrql() <= DISPATCH_LEVEL);

	KIRQL oldirql;
	KeAcquireSpinLock(&pdx->IoctlListLock, &oldirql);

	NTSTATUS status;

	if (*pIrp)
		status = STATUS_UNSUCCESSFUL;	// something already cached here
	else if (pdx->IoctlAbortStatus)
		status = pdx->IoctlAbortStatus;	// rejecting new IRPs for some reason
	else
		{						// try to cache IRP

		// Install a cancel routine and check for this IRP having already been
		// cancelled

		IoSetCancelRoutine(Irp, OnCancelPendingIoctl);
		if (Irp->Cancel && IoSetCancelRoutine(Irp, NULL))
			status = STATUS_CANCELLED;	// already cancelled

		// Put this IRP on our list of pending IOCTLs. Install a completion
		// routine to nullify the cache pointer. Note that AddDevice would have
		// failed if there were no PDO below us, so we know there's at least
		// one free stack location here.

		else
			{					// cache it
			IoMarkIrpPending(Irp);
			status = STATUS_PENDING;

			PIO_STACK_LOCATION stack = IoGetCurrentIrpStackLocation(Irp);
			stack->Parameters.Others.Argument1 = (PVOID) pIrp;
			IoSetCompletionRoutine(Irp, (PIO_COMPLETION_ROUTINE) OnCompletePendingIoctl, (PVOID) pdx, TRUE, TRUE, TRUE);
			IoSetNextIrpStackLocation(Irp);	// so our completion routine will get called
			PFILE_OBJECT fop = stack->FileObject;
			stack = IoGetCurrentIrpStackLocation(Irp);
			stack->DeviceObject = pdx->DeviceObject;	// so IoCancelIrp can give us right ptr
			stack->FileObject = fop;	// for cleanup

			InsertTailList(&pdx->PendingIoctlList, &Irp->Tail.Overlay.ListEntry);
			*pIrp = Irp;
			}					// cache it
		}						// try to cache IRP

	KeReleaseSpinLock(&pdx->IoctlListLock, oldirql);
	return status;
	}							// CacheControlRequest

///////////////////////////////////////////////////////////////////////////////

#pragma PAGEDCODE

VOID CleanupControlRequests(PDEVICE_EXTENSION pdx, NTSTATUS status, PFILE_OBJECT fop)
	{							// CleanupControlRequests
	LIST_ENTRY cancellist;
	InitializeListHead(&cancellist);

	// Create a list of IRPs that belong to the same file object

	KIRQL oldirql;
	KeAcquireSpinLock(&pdx->IoctlListLock, &oldirql);

	PLIST_ENTRY first = &pdx->PendingIoctlList;
	PLIST_ENTRY next;

	for (next = first->Flink; next != first; )
		{						// for each queued IRP
		PIRP Irp = CONTAINING_RECORD(next, IRP, Tail.Overlay.ListEntry);
		PIO_STACK_LOCATION stack = IoGetCurrentIrpStackLocation(Irp);

		// Follow the chain to the next IRP now (so that the next iteration of
		// the loop is properly setup whether we dequeue this IRP or not)

		PLIST_ENTRY current = next;
		next = next->Flink;

		// Skip this IRP if it's not for the same file object as the
		// current IRP_MJ_CLEANUP.

		if (fop && stack->FileObject != fop)
			continue;			// not for same file object

		// Set the CancelRoutine pointer to NULL and remove the IRP from the
		// queue.

		if (!IoSetCancelRoutine(Irp, NULL))
			continue;			// being cancelled right this instant
		RemoveEntryList(current);
		InsertTailList(&cancellist, current);
		}						// for each queued IRP

	// Release the spin lock. We're about to undertake a potentially time-consuming
	// operation that might conceivably result in a deadlock if we keep the lock.

	KeReleaseSpinLock(&pdx->IoctlListLock, oldirql);

	// Complete the selected requests.

	while (!IsListEmpty(&cancellist))
		{						// cancel selected requests
		next = RemoveHeadList(&cancellist);
		PIRP Irp = CONTAINING_RECORD(next, IRP, Tail.Overlay.ListEntry);
		Irp->IoStatus.Status = STATUS_CANCELLED;
		IoCompleteRequest(Irp, IO_NO_INCREMENT);
		}						// cancel selected requests
	}							// CleanupControlRequests

///////////////////////////////////////////////////////////////////////////////

#pragma LOCKEDCODE

VOID OnCancelPendingIoctl(PDEVICE_OBJECT fdo, PIRP Irp)
	{							// OnCancelPendingIoctl
	KIRQL oldirql = Irp->CancelIrql;
	IoReleaseCancelSpinLock(DISPATCH_LEVEL);
	PDEVICE_EXTENSION pdx = (PDEVICE_EXTENSION) fdo->DeviceExtension;

	// Remove the IRP from whatever queue it's on

	KeAcquireSpinLockAtDpcLevel(&pdx->IoctlListLock);
	RemoveEntryList(&Irp->Tail.Overlay.ListEntry);
	KeReleaseSpinLockFromDpcLevel(&pdx->IoctlListLock);

	// Complete the IRP

	Irp->IoStatus.Status = STATUS_CANCELLED;
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	}							// OnCancelPendingIoctl

///////////////////////////////////////////////////////////////////////////////

#pragma LOCKEDCODE

NTSTATUS OnCompletePendingIoctl(PDEVICE_OBJECT junk, PIRP Irp, PDEVICE_EXTENSION pdx)
	{							// OnCompletePendingIoctl
	KIRQL oldirql;
	KeAcquireSpinLock(&pdx->IoctlListLock, &oldirql);
	PIRP* pIrp = (PIRP*) IoGetCurrentIrpStackLocation(Irp)->Parameters.Others.Argument1;
	if (*pIrp == Irp)
		*pIrp = NULL;
	KeReleaseSpinLock(&pdx->IoctlListLock, oldirql);
	return STATUS_SUCCESS;
	}							// OnCompletePendingIoctl

///////////////////////////////////////////////////////////////////////////////

#pragma LOCKEDCODE

PIRP UncacheControlRequest(PDEVICE_EXTENSION pdx, PIRP* pIrp)
	{							// UncacheControlRequest
	ASSERT(KeGetCurrentIrql() <= DISPATCH_LEVEL);
	
	KIRQL oldirql;
	KeAcquireSpinLock(&pdx->IoctlListLock, &oldirql);

	PIRP Irp = (PIRP) InterlockedExchangePointer(pIrp, NULL);

	if (Irp)
		{						// an IRP was cached

		// Clear the cancel pointer for this IRP. Since both we and the
		// completion routine use a spin lock, it cannot happen that this
		// IRP pointer is suddenly invalid but the cache pointer cell
		// wasn't already NULL.

		if (IoSetCancelRoutine(Irp, NULL))
			{
			RemoveEntryList(&Irp->Tail.Overlay.ListEntry);	// N.B.: a macro!!
			}
		else
			Irp = NULL;			// currently being cancelled
		}						// an IRP was cached

	KeReleaseSpinLock(&pdx->IoctlListLock, oldirql);

	return Irp;
	}							// UncacheControlRequest

///////////////////////////////////////////////////////////////////////////////

VOID
IRQGenCancelIrp(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp
    )
{
	PDEVICE_EXTENSION extension = (PDEVICE_EXTENSION)DeviceObject->DeviceExtension;

	Irp->IoStatus.Information = 0;

	if (Irp == extension->PendingIRP)
	{
		extension->PendingIRP = NULL;
		Irp->IoStatus.Information = 1;
	}
	IoReleaseCancelSpinLock( Irp->CancelIrql );

	Irp->IoStatus.Status = STATUS_CANCELLED;
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
}